#####--------------KRIS PRUITT---------------------------------#####

#-------------------------------------------------------------------
#  		FUEL CELL (Combined Heat and Power) & GRID
#		Total Cost Objective
#-------------------------------------------------------------------

#-------------------------------------------------------------------
#  			Original Sets
#-------------------------------------------------------------------

####When running the full year instance, uncomment the next three set
####declarations and comment out the fourth set declaration.

#set N;					#set of all months
#set Tn{N};					#set of all hours in month n
#set T = union {n in N} Tn[n];		#set of all hours
set T;						#set of all hours


#-----------------------------------------------------------------------------
#		Underestimator set, bound parameters, and variables
#-----------------------------------------------------------------------------

set U;			#set of all underestimator vars

param GNhi{t in T};					#upper bound for GN
param GNlo{t in T};					#lower bound for GN
param HNhi{t in T};					#upper bound for HN
param HNlo{t in T};					#lower bound for HN
param GGhi{t in T};					#upper bound for GNG
param GGlo{t in T};					#lower bound for GNG
param HGhi{t in T};					#upper bound for HNG
param HGlo{t in T};					#lower bound for HNG
param GEhi{t in T};					#upper bound for GE
param GElo{t in T};					#lower bound for GE
param HEhi{t in T};					#upper bound for HE
param HElo{t in T};					#lower bound for HE
param WChi{t in T};					#upper bound for WC
param WClo{t in T};					#lower bound for WC
param WDhi{t in T};					#upper bound for WD
param WDlo{t in T};					#lower bound for WD
param Whi{t in T};					#upper bound for W
param Wlo{t in T};					#lower bound for W
param WXhi{t in T};					#upper bound for WX
param WXlo{t in T};					#lower bound for WX
param WYhi{t in T};					#upper bound for WY
param WYlo{t in T};					#lower bound for WY
param WAhi;						#upper bound on WA
param WAlo;						#lower bound on WA

#initial values for var bounds prior to tightening
param iGNhi{t in T};					#upper bound for GN
param iGNlo{t in T};					#lower bound for GN
param iHNhi{t in T};					#upper bound for HN
param iHNlo{t in T};					#lower bound for HN
param iGGhi{t in T};					#upper bound for GNG
param iGGlo{t in T};					#lower bound for GNG
param iHGhi{t in T};					#upper bound for HNG
param iHGlo{t in T};					#lower bound for HNG
param iGEhi{t in T};					#upper bound for GE
param iGElo{t in T};					#lower bound for GE
param iHEhi{t in T};					#upper bound for HE
param iHElo{t in T};					#lower bound for HE
param iWChi{t in T};					#upper bound for WC
param iWClo{t in T};					#lower bound for WC
param iWDhi{t in T};					#upper bound for WD
param iWDlo{t in T};					#lower bound for WD
param jWDlo{t in T};					#tight lower bound for WD
param iWhi{t in T};					#upper bound for W
param jWhi{t in T};					#tight upper bound for W
param iWlo{t in T};					#lower bound for W
param iWXhi{t in T};					#upper bound for WX
param iWXlo{t in T};					#lower bound for WX
param iWYhi{t in T};					#upper bound for WY
param jWYhi{t in T};					#tight upper bound for WY
param iWYlo{t in T};					#lower bound for WY
param iWAhi;						#upper bound on WA
param iWAlo;						#lower bound on WA

var UE{u in U,t in T};		#new variables to replace NL terms
					#trilinear then bilinear


#-----------------------------------------------------------------------------
#		Original parameters and variables
#-----------------------------------------------------------------------------

####When running the full year instance, uncomment the next cd declaration
####and comment out the second cd declaration.

#param cd{n in N};				#peak demand charge for electricity, month n
param cd;					#peak demand charge for electricity, month n

param ce{t in T};				#price of elec. from grid, hour t
param cg{t in T};				#energy charge for natural gas, hour t
param tau;					#time increment per period in hours
param de{t in T};				#electricity demand in period t
param dh{t in T};				#heat demand in period t
param gpx;					#max power capacity of power-only fcells
param hpx;					#max power capacity of CHP fcells
param vpx;					#max power capacity of PV cells
param pvf{t in T};				#forecasted percent max solar, period t
param gmt;					#min turn-down power-only fcells
param hmt;					#min turn-down CHP fcells
param gru;					#max ramp-up rate for power-only fcell
param grd;					#max ramp-down for power-only fcell
param hru;					#max ramp-up rate for CHP fcell
param hrd;					#max ramp-down for CHP fcell
param gaq;					#$/kW of power-only fuel cell
param haq;					#$/kW of CHP fuel cell
param vaq;					#$/kW of PV cell
param gom;					#variable o&m cost power fcell
param hom;					#variable o&m CHP fcell
param vom;					#variable o&m pv cell
param bom;					#variable o&m boiler
param waq;					#$/gal of tank
param wx;					#initial capacity of tank
param wxx;					#max tank capacity
param wd;					#decay rate of water temp
param we;					#heat x efficiency
param wu;					#max water temp
param wh;					#hot water delivery temp
param wr;					#average return water temp
param wl;					#water main temp
param ws;					#specific heat of water
param sx;					#max storage capacity of battery
param msc;					#min state-of-charge of battery
param scr;					#max charge rate for battery
param sdr;					#max discharge rate for battery
param sce;					#charge efficiency for battery
param sde;					#discharge efficiency for battery
param saq;					#$/kWh of battery
param fe;					#elec eff during start-up
param fh;					#CHP exhaust output per unit natural gas input
param ft;					#average temp of CHP exhaust
param fs;					#specific heat of exhaust
param fb;					#heat efficiency for boiler
param zt;					#carbon tax
param zu;					#carbon emission rate from grid
param zg;					#carbon emission rate of natural gas


####When running the full year instance, uncomment the next E declaration
####and comment out the second E declaration.

#var E{n in N} >= 0, :=max{t in Tn[n]}de[t];		#max supplied by grid in month n
var E >= 0, :=max{t in T}de[t];				#max supplied by grid in month n

var EP{t in T} >=0, :=de[t];				#power supplied by grid in period t
var ES{t in T} >=0, :=0;					#power sold back to grid in t
var GA >=0, integer, :=0;					#num power-only fuel cells acquired
var HA >=0, integer,:=0;					#num CHP fuel cells acquired
var VA >=0, integer,:=0;					#num PV cells acquired
var SA >=0, integer,:=0;					#num of batteries acquired
var WA >=WAlo, <=WAhi, :=wx;				#gallons for tank
var WB binary, :=0;						#1 if water tank acquired
var GN{t in T} integer,>=GNlo[t], <=GNhi[t], :=0;	#num power-only fcells turned on period t
var HN{t in T} integer,>=HNlo[t], <= HNhi[t], :=0;	#num CHP fcells turned on period t
var GY{t in T:t>1} >=0, :=0;				#num power fcells turned on t to t-1
var HY{t in T:t>1} >=0, :=0;				#num CHP fcells turned on t to t-1
var WY{t in T} binary,>=WYlo[t],<=WYhi[t], :=0;		#1 if water temp is above delivery temp in t
var WX{t in T} binary,>=WXlo[t],<=WXhi[t], :=0; 		#1 if water temp is above return temp in t
var P{t in T} >=0, :=0;					#power from power-only fuel cells in t
var CP{t in T} >=0, :=0;					#power from CHP fuel cells in t
var V{t in T} >=0, :=0;					#power from PV cells in t
var W{t in T} >=Wlo[t], <=Whi[t], :=wr;			#temp of water tank in t
var WC{t in T} >=WClo[t], <=WChi[t], :=0; 		#flowrate of CHP exhaust into tank in t
var WD{t in T} >=WDlo[t], <=WDhi[t], :=dh[t]/(ws*(wh-wr));	#flow rate of heated water in t
var SC{t in T} >=0, :=0;					#state-of-charge of batteries in t
var CR{t in T} >=0, :=0;					#charge rate in t
var DR{t in T} >=0, :=0;					#depletion rate in t
var GE{t in T} >=GElo[t], <=GEhi[t], :=fe;		#elec eff of power only SOFC
var HE{t in T} >=HElo[t], <=HEhi[t], :=fe;		#elec eff of CHP SOFC
var GNG{t in T} >=GGlo[t], <=GGhi[t], :=0;		#gas for power SOFCs
var HNG{t in T} >=HGlo[t], <=HGhi[t], :=0;		#gas for CHP SOFCs
var BNG{t in T} >=0, :=dh[t]/fb;				#gas for boiler




#################Underestimation Problem for Lower Bound#########################

#----------------------------------------------------------------------------------
#  				Objective Function 
#----------------------------------------------------------------------------------

####When running the full year instance, replace the final term of
####the objective (cd*E) with (sum{n in N}(cd[n]*E[n])).

minimize under_cost:	
gaq*gpx*GA + haq*hpx*HA + vaq*vpx*VA + saq*sx*SA + waq*(WA-wx) +
tau*sum{t in T}(gom*P[t] + hom*CP[t] + vom*V[t]) +
sum{t in T:t>1}((cg[t]+zt*zg)*(((gmt*gpx)/fe)*GY[t]+((hmt*hpx)/fe)*HY[t]))+ 
tau*sum{t in T}((bom*fb+cg[t]+zt*zg)*BNG[t]) +
tau*sum{t in T}((cg[t]+zt*zg)*(GNG[t]+HNG[t])) +
tau*sum{t in T}((ce[t]+zt*zu)*EP[t])- tau*sum{t in T}(ce[t]*ES[t]) +
cd*E;
 

#----------------------------------------------------------------------------------
#  			Original Constraints
#----------------------------------------------------------------------------------

####When running the full year instance, uncomment the first minmax constraint
####and the first netgen constraint, and comment out the second of each
####constraint.

#determine the maximum macrogrid demand in each month
#subject to minmax{n in N,t in Tn[n]}: E[n] >= EP[t];
subject to minmax{t in T}: E >= EP[t];

#cannot be a net generator each month
#subject to netgen{n in N}: sum{t in Tn[n]}ES[t] <= sum{t in Tn[n]}EP[t];
subject to netgen: sum{t in T}ES[t] <= sum{t in T}EP[t];


#power and heat demand met by DG, grid, and existing heating equipment
subject to demande{t in T}: P[t] + CP[t] + V[t] + (sde*DR[t]-CR[t]) + (EP[t]-ES[t]) = de[t];
subject to demandh{t in T}: -WD[t]*(ws*(wh-wr))*wl + (ws*(wh-wr))*UE[6,t] = dh[t]*(W[t]-wl-UE[7,t]+wh*WY[t]);

#max power from PV cells
subject to maxpowv{t in T}: V[t] <= pvf[t]*vpx*VA;

#max and min power from fuel cells
subject to pow_lo{t in T}: P[t] >= gmt*gpx*GN[t];
subject to pow_hi{t in T}: P[t] <= gpx*GN[t];
subject to pow_buyon{t in T}: GA >= GN[t]; 
subject to chp_lo{t in T}: CP[t] >= hmt*hpx*HN[t];
subject to chp_hi{t in T}: CP[t] <= hpx*HN[t];
subject to chp_buyon{t in T}: HA >= HN[t]; 

#ramp-up and ramp-down rate for fuel cells
subject to pow_rup{t in T:t<card(T)}: P[t+1] - P[t] <= tau*gru*GN[t+1];
subject to pow_rdown{t in T:t<card(T)}: P[t] - P[t+1] <= tau*grd*GN[t];
subject to chp_rup{t in T:t<card(T)}: CP[t+1] - CP[t] <= tau*hru*HN[t+1];
subject to chp_rdown{t in T:t<card(T)}: CP[t] - CP[t+1] <= tau*hrd*HN[t];

#number of fcells that sart up between t and t-1
subject to pow_oninc{t in T:t>1}: GN[t]-GN[t-1] <= GY[t];
subject to chp_oninc{t in T:t>1}: HN[t]-HN[t-1] <= HY[t];

#natural gas consumption
subject to ggas{t in T}: UE[1,t] = P[t];
subject to hgas{t in T}: UE[2,t] = CP[t];
subject to bgas{t in T}: BNG[t] = (ws/fb)*(wh*WD[t]-wh*UE[5,t]-UE[6,t]+UE[3,t]);

#electric efficiency of fuel cells
subject to pow_eff{t in T}: UE[8,t] = 0.61*GN[t]-0.02*P[t];
subject to chp_eff{t in T}: UE[9,t] = 0.61*HN[t]-0.02*CP[t];

#water tank acquisition
subject to tankacq1: WB <= HA;
subject to tankacq2: HNhi[1]*WB >= HA;

#temp of water tank
subject to tanktemp{t in T:t>1}: ws*UE[10,t] - ws*UE[10,t-1] + ws*wd*UE[4,t-1] = tau*we*fs*ft*WC[t-1] - tau*we*fs*UE[11,t-1] - tau*ws*UE[6,t-1] + tau*ws*wr*WD[t-1];
subject to temp1{t in T}: W[t] <= wr + (wu-wr)*WB;
subject to temp3{t in T}: W[t] <= wr + 0.1 + (wu-wr-0.1)*WX[t];
subject to temp4{t in T}: W[t] >= wr + 0.1*WX[t];
subject to temp5{t in T}: W[t] <= wh+(wu-wh)*WY[t];
subject to temp6{t in T}: W[t] >= wh-(wh-wr)*(1-WY[t]);
subject to temp7: W[1] = W[card(T)];

#heating water tank
subject to tankheat1{t in T}: WC[t] <= fh*HNG[t];

#state-of-charge of the battery
subject to soc{t in T:t>1}: SC[t] = SC[t-1] + tau*(sce*CR[t-1]-DR[t-1]);
subject to soc_init: SC[1] = SC[card(T)];

#state-of-charge max and min capacity
subject to st_upcap{t in T:t>1}: SC[t] <= sx*SA;
subject to st_locap{t in T:t>1}: SC[t] >= msc*sx*SA;

#battery max charge and deplete rates
subject to charge{t in T}: CR[t] <= scr*SA;
subject to deplete{t in T}: DR[t] <= sdr*SA;


#---------------------------------------------------------------------
#		Underestimator Constraints
#---------------------------------------------------------------------

#trilinear term constraints
subject to under5{t in T}: UE[3,t] >= (WD[t]*Wlo[t]*WYlo[t])+(WDlo[t]*W[t]*WYlo[t])+(WDlo[t]*Wlo[t]*WY[t])-2*(WDlo[t]*Wlo[t]*WYlo[t]);
subject to under6{t in T}: UE[3,t] >= (WD[t]*Whi[t]*WYhi[t])+(WDhi[t]*W[t]*WYlo[t])+(WDhi[t]*Wlo[t]*WY[t])-(WDhi[t]*Wlo[t]*WYlo[t])-(WDhi[t]*Whi[t]*WYhi[t]);
subject to under7{t in T}: UE[3,t] >= (WD[t]*Wlo[t]*WYlo[t])+(WDlo[t]*W[t]*WYhi[t])+(WDlo[t]*Whi[t]*WY[t])-(WDlo[t]*Whi[t]*WYhi[t])-(WDlo[t]*Wlo[t]*WYlo[t]);
subject to under8{t in T}: UE[3,t] >= (WD[t]*Whi[t]*WYlo[t])+(WDhi[t]*W[t]*WYhi[t])+(WDlo[t]*Whi[t]*WY[t])-(WDlo[t]*Whi[t]*WYlo[t])-(WDhi[t]*Whi[t]*WYhi[t]);
subject to under9{t in T}: UE[3,t] >= (WD[t]*Wlo[t]*WYhi[t])+(WDlo[t]*W[t]*WYlo[t])+(WDhi[t]*Wlo[t]*WY[t])-(WDhi[t]*Wlo[t]*WYhi[t])-(WDlo[t]*Wlo[t]*WYlo[t]);
subject to under10{t in T}: UE[3,t] >= (WD[t]*Wlo[t]*WYhi[t])+(WDlo[t]*W[t]*WYhi[t])+(WDhi[t]*Whi[t]*WY[t])-(WDlo[t]*Wlo[t]*WYhi[t])-(WDhi[t]*Whi[t]*WYhi[t]);
subject to under11{t in T}: UE[3,t] >= (WD[t]*Whi[t]*WYlo[t])+(WDhi[t]*W[t]*WYlo[t])+(WDlo[t]*Wlo[t]*WY[t])-(WDhi[t]*Whi[t]*WYlo[t])-(WDlo[t]*Wlo[t]*WYlo[t]);
subject to under12{t in T}: UE[3,t] >= (WD[t]*Whi[t]*WYhi[t])+(WDhi[t]*W[t]*WYhi[t])+(WDhi[t]*Whi[t]*WY[t])-2*(WDhi[t]*Whi[t]*WYhi[t]);

subject to under13{t in T}: UE[4,t] >= (WA*Wlo[t]*WXlo[t])+(WAlo*W[t]*WXlo[t])+(WAlo*Wlo[t]*WX[t])-2*(WAlo*Wlo[t]*WXlo[t]);
subject to under14{t in T}: UE[4,t] >= (WA*Whi[t]*WXhi[t])+(WAhi*W[t]*WXlo[t])+(WAhi*Wlo[t]*WX[t])-(WAhi*Wlo[t]*WXlo[t])-(WAhi*Whi[t]*WXhi[t]);
subject to under15{t in T}: UE[4,t] >= (WA*Wlo[t]*WXlo[t])+(WAlo*W[t]*WXhi[t])+(WAlo*Whi[t]*WX[t])-(WAlo*Whi[t]*WXhi[t])-(WAlo*Wlo[t]*WXlo[t]);
subject to under16{t in T}: UE[4,t] >= (WA*Whi[t]*WXlo[t])+(WAhi*W[t]*WXhi[t])+(WAlo*Whi[t]*WX[t])-(WAlo*Whi[t]*WXlo[t])-(WAhi*Whi[t]*WXhi[t]);
subject to under17{t in T}: UE[4,t] >= (WA*Wlo[t]*WXhi[t])+(WAlo*W[t]*WXlo[t])+(WAhi*Wlo[t]*WX[t])-(WAhi*Wlo[t]*WXhi[t])-(WAlo*Wlo[t]*WXlo[t]);
subject to under18{t in T}: UE[4,t] >= (WA*Wlo[t]*WXhi[t])+(WAlo*W[t]*WXhi[t])+(WAhi*Whi[t]*WX[t])-(WAlo*Wlo[t]*WXhi[t])-(WAhi*Whi[t]*WXhi[t]);
subject to under19{t in T}: UE[4,t] >= (WA*Whi[t]*WXlo[t])+(WAhi*W[t]*WXlo[t])+(WAlo*Wlo[t]*WX[t])-(WAhi*Whi[t]*WXlo[t])-(WAlo*Wlo[t]*WXlo[t]);
subject to under20{t in T}: UE[4,t] >= (WA*Whi[t]*WXhi[t])+(WAhi*W[t]*WXhi[t])+(WAhi*Whi[t]*WX[t])-2*(WAhi*Whi[t]*WXhi[t]);

#bilinear term constraints
subject to under1{t in T}: UE[1,t] >= (GGlo[t]*GE[t])+(GElo[t]*GNG[t])-(GGlo[t]*GElo[t]);
subject to under2{t in T}: UE[1,t] >= (GGhi[t]*GE[t])+(GEhi[t]*GNG[t])-(GGhi[t]*GEhi[t]);
subject to under49{t in T}: UE[1,t] <= (GGhi[t]*GE[t])+(GElo[t]*GNG[t])-(GGhi[t]*GElo[t]);
subject to under50{t in T}: UE[1,t] <= (GGlo[t]*GE[t])+(GEhi[t]*GNG[t])-(GGlo[t]*GEhi[t]);

subject to under3{t in T}: UE[2,t] >= (HGlo[t]*HE[t])+(HElo[t]*HNG[t])-(HGlo[t]*HElo[t]);
subject to under4{t in T}: UE[2,t] >= (HGhi[t]*HE[t])+(HEhi[t]*HNG[t])-(HGhi[t]*HEhi[t]);
subject to under51{t in T}: UE[2,t] <= (HGhi[t]*HE[t])+(HElo[t]*HNG[t])-(HGhi[t]*HElo[t]);
subject to under52{t in T}: UE[2,t] <= (HGlo[t]*HE[t])+(HEhi[t]*HNG[t])-(HGlo[t]*HEhi[t]);

subject to under21{t in T}: UE[5,t] >= (WDlo[t]*WY[t])+(WYlo[t]*WD[t])-(WDlo[t]*WYlo[t]);
subject to under22{t in T}: UE[5,t] >= (WDhi[t]*WY[t])+(WYhi[t]*WD[t])-(WDhi[t]*WYhi[t]);
subject to under23{t in T}: UE[5,t] <= (WDhi[t]*WY[t])+(WYlo[t]*WD[t])-(WDhi[t]*WYlo[t]);
subject to under24{t in T}: UE[5,t] <= (WDlo[t]*WY[t])+(WYhi[t]*WD[t])-(WDlo[t]*WYhi[t]);

subject to under25{t in T}: UE[6,t] >= (WDlo[t]*W[t])+(Wlo[t]*WD[t])-(WDlo[t]*Wlo[t]);
subject to under26{t in T}: UE[6,t] >= (WDhi[t]*W[t])+(Whi[t]*WD[t])-(WDhi[t]*Whi[t]);
subject to under27{t in T}: UE[6,t] <= (WDhi[t]*W[t])+(Wlo[t]*WD[t])-(WDhi[t]*Wlo[t]);
subject to under28{t in T}: UE[6,t] <= (WDlo[t]*W[t])+(Whi[t]*WD[t])-(WDlo[t]*Whi[t]);

subject to under29{t in T}: UE[7,t] >= (WYlo[t]*W[t])+(Wlo[t]*WY[t])-(WYlo[t]*Wlo[t]);
subject to under30{t in T}: UE[7,t] >= (WYhi[t]*W[t])+(Whi[t]*WY[t])-(WYhi[t]*Whi[t]);
subject to under31{t in T}: UE[7,t] <= (WYhi[t]*W[t])+(Wlo[t]*WY[t])-(WYhi[t]*Wlo[t]);
subject to under32{t in T}: UE[7,t] <= (WYlo[t]*W[t])+(Whi[t]*WY[t])-(WYlo[t]*Whi[t]);

subject to under33{t in T}: UE[8,t] >= (GNlo[t]*GE[t])+(GElo[t]*GN[t])-(GNlo[t]*GElo[t]);
subject to under34{t in T}: UE[8,t] >= (GNhi[t]*GE[t])+(GEhi[t]*GN[t])-(GNhi[t]*GEhi[t]);
subject to under35{t in T}: UE[8,t] <= (GNhi[t]*GE[t])+(GElo[t]*GN[t])-(GNhi[t]*GElo[t]);
subject to under36{t in T}: UE[8,t] <= (GNlo[t]*GE[t])+(GEhi[t]*GN[t])-(GNlo[t]*GEhi[t]);

subject to under37{t in T}: UE[9,t] >= (HNlo[t]*HE[t])+(HElo[t]*HN[t])-(HNlo[t]*HElo[t]);
subject to under38{t in T}: UE[9,t] >= (HNhi[t]*HE[t])+(HEhi[t]*HN[t])-(HNhi[t]*HEhi[t]);
subject to under39{t in T}: UE[9,t] <= (HNhi[t]*HE[t])+(HElo[t]*HN[t])-(HNhi[t]*HElo[t]);
subject to under40{t in T}: UE[9,t] <= (HNlo[t]*HE[t])+(HEhi[t]*HN[t])-(HNlo[t]*HEhi[t]);

subject to under41{t in T}: UE[10,t] >= (WAlo*W[t])+(Wlo[t]*WA)-(WAlo*Wlo[t]);
subject to under42{t in T}: UE[10,t] >= (WAhi*W[t])+(Whi[t]*WA)-(WAhi*Whi[t]);
subject to under43{t in T}: UE[10,t] <= (WAhi*W[t])+(Wlo[t]*WA)-(WAhi*Wlo[t]);
subject to under44{t in T}: UE[10,t] <= (WAlo*W[t])+(Whi[t]*WA)-(WAlo*Whi[t]);

subject to under45{t in T}: UE[11,t] >= (WClo[t]*W[t])+(Wlo[t]*WC[t])-(WClo[t]*Wlo[t]);
subject to under46{t in T}: UE[11,t] >= (WChi[t]*W[t])+(Whi[t]*WC[t])-(WChi[t]*Whi[t]);
subject to under47{t in T}: UE[11,t] <= (WChi[t]*W[t])+(Wlo[t]*WC[t])-(WChi[t]*Wlo[t]);
subject to under48{t in T}: UE[11,t] <= (WClo[t]*W[t])+(Whi[t]*WC[t])-(WClo[t]*Whi[t]);




##############Linearization Problem for Upper Bound###########################

#-----------------------------------------------------------------------------------
#  		New parameters and variables for linearization problem
#-----------------------------------------------------------------------------------

param dGE{t in T};
param dHE{t in T};
param dW{t in T};
param dWX{t in T};
param dWY{t in T};


####When running the full year instance, uncomment the next dE declaration
####and comment out the second dE declaration.

#var dE{n in N} >= 0;		#max supplied by grid in month n
var dE >= 0;				#max supplied by grid in month n

var dEP{t in T} >=0;			#power supplied by grid in period t
var dES{t in T} >=0;			#power sold back to grid in t
var dWA >=0;				#volume of water tank
var dWB binary;			#1 if water tank acquired
var dWD{t in T} >=0;			#flowrate of water from tank
var dWC{t in T} >=0;			#flowrate of exhaust into tank
var dBNG{t in T} >=0;		#nat gas into boiler
var dGA >=0, integer;		#num power-only fcells acquired
var dHA >=0, integer;		#num CHP fcells acquired
var dVA >=0, integer;		#num PV cells acquired
var dSA >=0, integer;		#num batteries acquired
var dGN{t in T} >=0, integer;	#num power-only fcells turned on period t
var dHN{t in T} >=0, integer;	#num CHP fcells turned on period t
var dGY{t in T:t>1} >=0;		#num power fcells turned on t to t-1
var dHY{t in T:t>1} >=0;		#num CHP fcells turned on t to t-1
var dP{t in T} >=0;			#power from power-only fuel cells in t
var dCP{t in T} >=0;			#power from CHP fuel cells in t
var dV{t in T} >=0;			#power from PV cells in t
var dSC{t in T} >=0;			#state-of-charge of batteries in t
var dCR{t in T} >=0;			#charge rate in t
var dDR{t in T} >=0;			#depletion rate in t
var dGNG{t in T} >=0;		#gas for power fcells
var dHNG{t in T} >=0;		#gas for CHP fcells


#---------------------------------------------------------
#		Objective Function
#---------------------------------------------------------

####When running the full year instance, replace the final term of
####the objective (cd*dE) with (sum{n in N}(cd[n]*dE[n])).

minimize heuristic_cost:	
gaq*gpx*dGA + haq*hpx*dHA + vaq*vpx*dVA + saq*sx*dSA + 
waq*(dWA-wx) + 
tau*sum{t in T}((bom*fb+cg[t]+zt*zg)*dBNG[t])+
tau*sum{t in T}(gom*dP[t] + hom*dCP[t] + vom*dV[t]) +
sum{t in T:t>1}((cg[t]+zt*zg)*(((gmt*gpx)/fe)*dGY[t]+((hmt*hpx)/fe)*dHY[t]))+ 
tau*sum{t in T}((cg[t]+zt*zg)*(dGNG[t]+dHNG[t])) +
tau*sum{t in T}((ce[t]+zt*zu)*dEP[t])- tau*sum{t in T}(ce[t]*dES[t]) +
cd*dE; 


#--------------------------------------------------------
#		Constraints
#--------------------------------------------------------

####When running the full year instance, uncomment the first dminmax constraint
####and the first dnetgen constraint, and comment out the second of each
####constraint.

#determine the maximum macrogrid demand in each month
#subject to dminmax{n in N,t in Tn[n]}: dE[n] >= dEP[t];
subject to dminmax{t in T}: dE >= dEP[t];

#cannot be a net generator each month
#subject to dnetgen{n in N}: sum{t in Tn[n]}dES[t] <= sum{t in Tn[n]}dEP[t];
subject to dnetgen: sum{t in T}dES[t] <= sum{t in T}dEP[t];


#power and heat demand met by DG and grid
subject to ddemande{t in T}: dP[t] + dCP[t] + dV[t] + (sde*dDR[t]-dCR[t]) + (dEP[t]-dES[t]) = de[t];
subject to ddemandh{t in T}: -dWD[t]*(ws*(wh-wr))*wl + (ws*(wh-wr))*dW[t]*dWD[t] = dh[t]*(dW[t]-wl-dW[t]*dWY[t]+wh*dWY[t]);

#max power from PV cells
subject to dmaxpowv{t in T}: dV[t] <= pvf[t]*vpx*dVA;

#max and min power from fuel cells
subject to dpow_lo{t in T}: dP[t] >= gmt*gpx*dGN[t];
subject to dpow_hi{t in T}: dP[t] <= gpx*dGN[t];
subject to dpow_buyon{t in T}: dGA >= dGN[t]; 
subject to dchp_lo{t in T}: dCP[t] >= hmt*hpx*dHN[t];
subject to dchp_hi{t in T}: dCP[t] <= hpx*dHN[t];
subject to dchp_buyon{t in T}: dHA >= dHN[t]; 

#ramp-up and ramp-down rate for fuel cells
subject to dpow_rup{t in T:t<card(T)}: dP[t+1] - dP[t] <= tau*gru*dGN[t+1];
subject to dpow_rdown{t in T:t<card(T)}: dP[t] - dP[t+1] <= tau*grd*dGN[t];
subject to dchp_rup{t in T:t<card(T)}: dCP[t+1] - dCP[t] <= tau*hru*dHN[t+1];
subject to dchp_rdown{t in T:t<card(T)}: dCP[t] - dCP[t+1] <= tau*hrd*dHN[t];

#number of fcells that start up between t and t-1
subject to dpow_oninc{t in T:t>1}: dGN[t]-dGN[t-1] <= dGY[t];
subject to dchp_oninc{t in T:t>1}: dHN[t]-dHN[t-1] <= dHY[t];

#natural gas consumption
subject to dggas{t in T}: dGE[t]*dGNG[t] = dP[t];
subject to dhgas{t in T}: dHE[t]*dHNG[t] = dCP[t];
subject to dbgas{t in T}: dBNG[t] = (ws/fb)*(wh*dWD[t]-wh*dWY[t]*dWD[t]-dW[t]*dWD[t]+dWD[t]*dW[t]*dWY[t]);

#new constraints for efficiency
subject to dpow_eff{t in T}: dGE[t]*dGN[t] = 0.61*dGN[t]-0.02*dP[t];
subject to dchp_eff{t in T}: dHE[t]*dHN[t] = 0.61*dHN[t]-0.02*dCP[t];

#temp of water tank
subject to dtanktemp{t in T:t>1}: ws*dW[t]*dWA - ws*dW[t-1]*dWA + ws*wd*dWA*dWX[t-1]*dW[t-1] = tau*we*fs*ft*dWC[t-1] - tau*we*fs*dW[t-1]*dWC[t-1] - tau*ws*dW[t-1]*dWD[t-1] + tau*ws*wr*dWD[t-1];
subject to dtemp1{t in T}: dW[t] <= wr + (wu-wr)*dWB;

#heating water tank
#subject to tankheat1{t in T}: WC[t] <= fh*(CP[t]/HE[t]);
subject to dtankheat1{t in T}: dWC[t] <= fh*dHNG[t];

#water tank acquisition
subject to dtankacq1: dWB <= dHA;
subject to dtankacq2: HNhi[1]*dWB >= dHA;
subject to dtanksize1: dWA >= wx;
subject to dtanksize2: dWA <= wxx;

#state-of-charge of the battery
subject to dsoc{t in T:t>1}: dSC[t] = dSC[t-1] + tau*(sce*dCR[t-1]-dDR[t-1]);
subject to dsoc_init: dSC[1] = dSC[card(T)];

#state-of-charge max and min capacity
subject to dst_upcap{t in T:t>1}: dSC[t] <= sx*dSA;
subject to dst_locap{t in T:t>1}: dSC[t] >= msc*sx*dSA;

#battery max charge and deplete rates
subject to dcharge{t in T}: dCR[t] <= scr*dSA;
subject to ddeplete{t in T}: dDR[t] <= sdr*dSA;














